@@ -86,42 +86,45 @@ DEFUN_NEW("SETVV",object,fSsetvv,SI
 DEF_ORDINARY("%MEMORY",sSPmemory,SI,"");
 DEF_ORDINARY("%INIT",sSPinit,SI,"");
 
-void Lidentity(void);
+/* void Lidentity(void); */
 void
-init_cmpaux(void)
+gcl_init_cmpaux(void)
 {
 
 
 	/* real one defined in predlib.lsp, need this for bootstrap */
-	make_si_function("WARN-VERSION",Lidentity);
+/* 	make_si_function("WARN-VERSION",Lidentity); */
 	
 }
 
   
-int
-ifloor(int x, int y)
-{
-  if (y == 0) {
-    FEerror("Zero divizor", 0);
-    return 0;
-  }
-  if (y > 0) {
-    if (x >= 0)
-      return(x/y);
-    else
-      return(-((-x+y-1))/y);
-  }
-  if (x >= 0)
-    return(-((x-y-1)/(-y)));
-  else
-    return((-x)/(-y));
-}
+/* Now inlined directly by optimizer  */
+/* int */
+/* ifloor(int x, int y) */
+/* { */
+/*   if (y == 0) { */
+/*     FEerror("Zero divizor", 0); */
+/*     return 0; */
+/*   } */
+/*   if (y > 0) { */
+/*     if (x >= 0) */
+/*       return(x/y); */
+/*     else */
+      /* FIXME, deal with possible overflow here*/
+/*       return(-((-x-1))/y-1); */
+/*   } */
+/*   if (x >= 0) */
+      /* FIXME, deal with possible overflow here*/
+/*     return(-((x-1)/(-y))-1); */
+/*   else */
+/*     return((-x)/(-y)); */
+/* } */
 
-int
-imod(int x, int y)
-{
-  return(x - ifloor(x, y)*y);
-}
+/* int */
+/* imod(int x, int y) */
+/* { */
+/*   return(x - ifloor(x, y)*y); */
+/* } */
 
 /* static void */
 /* set_VV(object *, int, object); */
@@ -433,7 +436,7 @@ char *s;
 #endif
 
 void
-init_or_load1(void (*fn)(void),char *file)
+gcl_init_or_load1(void (*fn)(void),char *file)
 {int n=strlen(file);
  if (file[n-1]=='o')
    { object memory;
@@ -470,7 +473,7 @@ function.")
 
   int i;
 
-  i=init_cmp_anon();
+  i=gcl_init_cmp_anon();
   if (i<0) 
     FEerror("No such anonymous function",0);
 
